@@ -1,8 +1,8 @@
-/* Copyright (C) 1998,1999 Nikos Mavroyanopoulos
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+/* Copyright (C) 1998,1999 Nikos Mavroyanopoulos This library is free
+ * software; you can redistribute it and/or modify it under the terms of the
+ * GNU Library General Public License as published by the Free Software
+ * Foundation; either version 2 of the License, or (at your option) any
+ * later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,9 +29,12 @@
 #include <safer.h>
 #include <cast.h>
 #include <tean.h>
-#include <twofish.h>
+#include <twofish/aes.h>
 #include <xmemory.h>
 #include <rc2.h>
+#include <rc6.h>
+#include <idea.h>
+#include <sha1.h> /* for gen_key */
 
 #define MAX_THREADS 256 /* After that pointers are being reused */
 
@@ -51,7 +54,9 @@ static int cur_thread=0; /* No threads */
 static struct cast_key* ckey=NULL; /* For cast */
 static blf_ctx *c=NULL; /* For Blowfish */
 
-static TWOFISH_context *CT=NULL; /* For Twofish */
+static cipherInstance *CI=NULL; /* twofish */
+static keyInstance *KI=NULL;
+
 
 static char **kn1=NULL; /* For DES */
 static char **kn2=NULL;
@@ -59,6 +64,13 @@ static char **kn3=NULL; /* For tripleDES */
 static word8** saferkey=NULL; /* For safer64 and 128 */
 static word16 **rc2_key; /* For rc2 */
 
+#ifdef NON_FREE
+static word16 **idea_ikey;
+static word16 **idea_key; /* For idea */
+
+static word32 **rc6_skey; /* For RC6 */
+#endif
+
 /* For CBC mode */
 static word32 **previous_ciphertext;
 static word32 **previous_plaintext;
@@ -92,10 +104,22 @@ int get_block_size(int algorithm)
       return 8;
       break;
 
-   case TWOFISH:
-      return 16;
+#ifdef NON_FREE
+   case IDEA:
+      return 8;
       break;
       
+   case RC6:
+      return 16;
+      break;
+#endif
+
+   case TWOFISH128:
+   case TWOFISH192:
+   case TWOFISH256:
+      return 16;
+      break;
+
    case ThreeWAY:
       return 12;
       break;
@@ -128,11 +152,22 @@ int get_key_size(int algorithm) /* In bytes */
       return 128;
       break;
 
+#ifdef NON_FREE
+   case RC6:
+      return RC6_BKEY; /* 32 */
+      break;
+#endif
+
+   case TWOFISH192:
+      return 24;
+      break;
+
    case GOST:
+   case TWOFISH256:
       return 32;
       break;
 
-   case TWOFISH:
+   case TWOFISH128:
       return 16;
       break;
 
@@ -143,6 +178,9 @@ int get_key_size(int algorithm) /* In bytes */
    case SAFER128:
    case CAST128:
    case TEAN:
+#ifdef NON_FREE
+   case IDEA:
+#endif
       return 16;
       break;
 
@@ -157,8 +195,10 @@ int init_mcrypt(const int algorithm, int thread, void * key, int lenofkey)
 {
 
 char *keyword1, *keyword2, *keyword3; /* For 3DES and safer*/
-unsigned char *skey=NULL;
-word8 twokey[16]; /* 128 bits */
+unsigned char *safer_skey=NULL;
+char twokey[68];
+int i,j;
+/*int a,b,cnt;*/
 
  if (used_all_threads==0) {
   saferkey=mxrealloc(saferkey, (thread+1)*sizeof(char*));
@@ -166,28 +206,47 @@ word8 twokey[16]; /* 128 bits */
   kn2=mxrealloc(kn2, (thread+1)*sizeof(char*));
   kn3=mxrealloc(kn3, (thread+1)*sizeof(char*));
   c=mxrealloc(c, (thread+1) * sizeof(blf_ctx));
-  CT=mxrealloc(CT, (thread+1) * sizeof(TWOFISH_context));
+  CI=mxrealloc(CI, (thread+1) * sizeof(cipherInstance));
+  KI=mxrealloc(KI, (thread+1) * sizeof(keyInstance));
   ckey=mxrealloc(ckey, (thread+1) * sizeof(struct cast_key));
   keyword_given=mxrealloc(keyword_given, (thread+1)*sizeof(char*));
   rc2_key=mxrealloc(rc2_key, (thread+1)*sizeof(word16*));
+
+#ifdef NON_FREE
+  rc6_skey=mxrealloc(rc6_skey, (thread+1)*sizeof(word32*));
+  idea_key=mxrealloc(idea_key, (thread+1)*sizeof(word16*));
+  idea_ikey=mxrealloc(idea_ikey, (thread+1)*sizeof(word16*));
+#endif
+
   algorithm_given=mxrealloc(algorithm_given, (thread+1)*sizeof(int));
  }
-  rc2_key[thread]=malloc(64*sizeof(word16));  
+  
   if (get_key_size(algorithm) < lenofkey) lenofkey=get_key_size(algorithm);
 
   check_pointer[thread]=1;  
   algorithm_given[thread]=algorithm;
   keyword_given[thread]=mxcalloc(1, get_key_size(algorithm));
   memmove(keyword_given[thread], key, lenofkey);
-  skey=keyword_given[thread];
+  safer_skey=keyword_given[thread];
 
 
    switch(algorithm) {
   
    case DES:
+/*
+		for(cnt = 0; cnt < 8; cnt++){
+			b = 0;
+			for(a=0;a<7;a++)
+				if( keyword_given[thread][cnt] & (1 << a))
+					b++;
+			if((b & 1) == 0)
+				keyword_given[thread][cnt] |= 0x80;
+			else
+				keyword_given[thread][cnt] &= ~0x80;
+		}
+*/
       desinit(0);
       kn1[thread]=mxmalloc(8*16*sizeof(char));
-      
       des_setkey( (void*) kn1[thread], (void *) keyword_given[thread]);
       return thread;
       break;
@@ -208,17 +267,35 @@ word8 twokey[16]; /* 128 bits */
       break; 
 
    case RC2:
+      if (lenofkey==0) lenofkey=1; /* So 1 byte is used */
+      rc2_key[thread]=mxmalloc(64*sizeof(word16));  
       rc2_keyschedule( rc2_key[thread], (void*)keyword_given[thread], lenofkey, 1024);
       return thread;
       break; 
 
+#ifdef NON_FREE
+   case IDEA:
+      idea_key[thread]=mxmalloc(52*sizeof(word16));
+      idea_ikey[thread]=mxmalloc(52*sizeof(word16));
+      Idea_ExpandUserKey( (void*)keyword_given[thread], idea_key[thread]);
+      Idea_InvertKey( idea_key[thread], idea_ikey[thread]);
+      return thread;
+      break; 
+
+   case RC6:
+      rc6_skey[thread]=mxmalloc(RC6_FACTOR*sizeof(word32));  
+      rc6_schedule( (void*)keyword_given[thread], rc6_skey[thread]);
+      return thread;
+      break; 
+#endif
+
    case SAFER128:
       Safer_Init_Module();
       keyword1=mxmalloc(8); keyword2=mxmalloc(8);
       saferkey[thread]=mxmalloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
       
-      memmove(keyword1, skey, 8);
-      memmove(keyword2, &skey[8], 8);
+      memmove(keyword1, safer_skey, 8);
+      memmove(keyword2, &safer_skey[8], 8);
       Safer_Expand_Userkey( (unsigned char*)keyword1, (unsigned char*)keyword2, SAFER_SK128_DEFAULT_NOF_ROUNDS, 1, saferkey[thread]);
       /* 8 rounds and secure key schedule */
       free(keyword1); free(keyword2);
@@ -260,12 +337,27 @@ word8 twokey[16]; /* 128 bits */
       return thread;
       break;
 
-   case TWOFISH:
-
-      Bzero( twokey, sizeof(twokey));
-      memmove( twokey, keyword_given[thread], 16); /* 128 bit only */
-      twofish_setkey ( &CT[thread], twokey, 16);
+   case TWOFISH128:
+   case TWOFISH192:
+   case TWOFISH256:
+   
+      Bzero( &KI[cur_thread], sizeof(keyInstance));
+      Bzero( &CI[cur_thread], sizeof(cipherInstance));
+      cipherInit( &CI[cur_thread], MODE_ECB,NULL);
 
+      j=0;
+      for (i=0;i<lenofkey;i++) {
+          sprintf( &twokey[j], "%.2x", (unsigned char) keyword_given[cur_thread][i]);
+          j+=2;
+       }
+      for (;i<get_key_size(algorithm_given[cur_thread]);i++) {
+          sprintf( &twokey[j], "%.2x", 0);
+          j+=2;
+       }
+                                   
+       j=makeKey ( &KI[cur_thread], DIR_ENCRYPT, get_key_size(algorithm_given[cur_thread])*8,  twokey);
+       if (j<0) { return (-1); }
+                 
       return thread;
       break;
 
@@ -311,6 +403,18 @@ int mcrypt(int thread, void *plaintext)
       rc2_encrypt( rc2_key[thread], (void*) plaintext, (void*) plaintext);
       return 0;
       break;
+
+#ifdef NON_FREE   
+   case IDEA:
+      Idea_Crypt( (void*) plaintext, (void*) plaintext, idea_key[thread]);
+      return 0;
+      break;
+      
+   case RC6:
+      rc6_encrypt( (void*) plaintext, rc6_skey[thread]);
+      return 0;
+      break;
+#endif
             
    case DES:
       endes ( (void*)kn1[thread], plaintext);
@@ -329,8 +433,10 @@ int mcrypt(int thread, void *plaintext)
       return 0;
       break;
 
-   case TWOFISH:
-      twofish_encrypt(&CT[thread], plaintext, plaintext); /* 16 bytes */ 
+   case TWOFISH128:
+   case TWOFISH192:
+   case TWOFISH256:
+      blockEncrypt(&CI[thread], &KI[thread], plaintext, 16*8, plaintext);
       return 0;
       break;
 
@@ -373,11 +479,22 @@ int mdecrypt(int thread, void *plaintext)
       break;
 
    case RC2:
-
       rc2_decrypt( rc2_key[thread], (void*) plaintext, (void*) plaintext);
       return 0;
       break;
 
+#ifdef NON_FREE
+   case IDEA:
+      Idea_Crypt( (void*) plaintext, (void*) plaintext, idea_ikey[thread]);
+      return 0;
+      break;
+
+   case RC6:
+      rc6_decrypt( (void*) plaintext, rc6_skey[thread]);
+      return 0;
+      break;
+#endif
+
    case DES:
       dedes ( (char (*)[8])kn1[thread], plaintext);
       return 0;
@@ -395,8 +512,10 @@ int mdecrypt(int thread, void *plaintext)
       return 0;
       break;
 
-   case TWOFISH:
-      twofish_decrypt(&CT[thread], plaintext, plaintext); /* 16 bytes */
+   case TWOFISH128:
+   case TWOFISH192:
+   case TWOFISH256:
+      blockDecrypt(&CI[thread], &KI[thread], plaintext, 16*8, plaintext);
       return 0;
       break;
 
@@ -424,6 +543,7 @@ int mdecrypt(int thread, void *plaintext)
 int end_mcrypt(int td)
 {
 
+Bzero(keyword_given[td], get_key_size(algorithm_given[td]));
 free(keyword_given[td]); 
 check_pointer[td]=0;
 
@@ -431,31 +551,62 @@ check_pointer[td]=0;
   
    case DES:
 	desdone();
+        Bzero(kn1[td], 8*16*sizeof(char));
 	free(kn1[td]);
       break;
 
    case TripleDES:
 	desdone();
+        Bzero(kn1[td], 8*16*sizeof(char));
+        Bzero(kn2[td], 8*16*sizeof(char));
+        Bzero(kn3[td], 8*16*sizeof(char));        
 	free(kn1[td]);
 	free(kn2[td]);
 	free(kn3[td]);
       break;
 
    case RC2:
+        Bzero(rc2_key[td], 64*sizeof(word16));
    	free(rc2_key[td]);
       break;
-      
+   
+#ifdef NON_FREE
+   case IDEA:
+        Bzero(idea_key[td], 52*sizeof(word16));
+        Bzero(idea_key[td], 52*sizeof(word16));
+        free(idea_key[td]);
+        free(idea_ikey[td]);
+        break;
+        
+   case RC6:
+        Bzero(rc6_skey[td], RC6_FACTOR*sizeof(word32));
+   	free(rc6_skey[td]);
+      break;
+#endif
+
+   case TWOFISH128:
+   case TWOFISH192:
+   case TWOFISH256:
+        Bzero(&CI[td], sizeof(cipherInstance));
+        Bzero(&KI[td], sizeof(keyInstance));
+        break;
+
    case BLOWFISH:
+        Bzero(&c[td], sizeof(blf_ctx));
+        break;
+        
    case TEAN:   
 	break;
 
    case GOST:
    case ThreeWAY:
    case CAST128:
+         Bzero(&ckey[td], sizeof(struct cast_key));
         break;
         
    case SAFER64:
    case SAFER128:
+        Bzero(saferkey[td], 1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
 	free(saferkey[td]);   
 	break;
       break;
@@ -949,14 +1100,29 @@ char *x=mxmalloc(20);
     strcpy(x,"DES");
     return x;
     break;
+
    case RC2:
     strcpy(x,"RC2");
     return x;
     break;
+
+#ifdef NON_FREE
+   case IDEA:
+    strcpy(x,"IDEA");
+    return x;
+    break;
+    
+   case RC6:
+    strcpy(x,"RC6");
+    return x;
+    break;
+#endif
+    
    case TripleDES:
     strcpy(x,"3DES");
     return x;
     break;
+    
    case ThreeWAY:
     strcpy(x,"3-WAY");
     return x;
@@ -967,24 +1133,41 @@ char *x=mxmalloc(20);
     strcpy(x,"Crypt");
     return x;
     break;
+
+   case DES_COMPAT:
+    strcpy(x,"DES-COMPAT");
+    return x;
+    break;
 #endif
 
    case BLOWFISH:
     strcpy(x,"Blowfish");
     return x;
     break;
-   case TWOFISH:
-    strcpy(x,"Twofish");
+    
+   case TWOFISH128:
+    strcpy(x,"Twofish-128");
     return x;
-    break;    
+    break;
+   case TWOFISH192:
+    strcpy(x,"Twofish-192");
+    return x;
+    break;
+   case TWOFISH256:
+    strcpy(x,"Twofish-256");
+    return x;
+    break;
+    
    case GOST:
     strcpy(x,"GOST");
     return x;
     break;
+    
    case SAFER64:
     strcpy(x,"SAFER-SK64");
     return x;
     break;
+    
    case SAFER128:
     strcpy(x,"SAFER-SK128");
     return x;
@@ -1001,3 +1184,99 @@ char *x=mxmalloc(20);
 return NULL;
 
 }
+
+
+int is_ok_algorithm(int algorithm) {
+
+  switch(algorithm) {
+   case DES:
+   case RC2:
+#ifdef NON_FREE
+   case IDEA:
+   case RC6:
+#endif
+   case TripleDES:
+   case ThreeWAY:
+#ifdef MCRYPT2
+   case CRYPT:
+   case DES_COMPAT:
+#endif
+   case BLOWFISH:
+   case TWOFISH128:
+   case TWOFISH192:
+   case TWOFISH256:
+   case GOST:
+   case SAFER64:
+   case SAFER128:
+   case CAST128:
+   case TEAN:
+      return 0;
+      break;
+      
+   default:
+      return 1;
+      break;
+    
+  }
+
+}
+
+int is_ok_mode(int mode) { /* if this mode exists returns 0 */
+
+  switch(mode) {
+   case OFB:
+   case CFB:
+   case CBC:
+   case ECB:
+      return 0;
+      break;
+      
+   default:
+      return 1;
+      break;
+    
+  }
+} 
+
+
+void gen_key(void *keyword, void *salt, size_t salt_size, size_t given_size,
+             unsigned char *password)
+{
+        unsigned char   key[MAX_KEY_LEN];
+        SHA1_CTX        ctx;
+        unsigned char   *digest;  
+        int             plen    = strlen((void*)password);
+        size_t          size    = given_size;
+        unsigned char   *cp     = key,
+                        *cp0    = key;
+        int             salt_z  = 0;
+        
+        if (salt==NULL) { 
+          if (salt_size==0) salt_size=1;
+          salt=mxmalloc(salt_size);
+          Bzero (salt, salt_size); /* In bare mode salt is nulls */
+          salt_z=1;
+        }                         /* remember? no headers there */
+        
+        while (1) {
+                SHA1Init(&ctx);
+                SHA1Update(&ctx, salt, salt_size);
+                SHA1Update(&ctx, password, plen);
+                if (cp - cp0 > 0)
+                        SHA1Update(&ctx, cp0, cp - cp0);
+                digest = SHA1Final( &ctx);
+                if (size > 20) { 
+                        memmove(cp, digest, 20);
+                        free(digest);
+                        size -= 20;
+                        cp += 20;
+                } else {
+                        memmove(cp, digest, size);
+                        free(digest);
+                        break;
+                }
+        }
+        memmove (keyword, key, given_size);
+        if (salt_z==1) free(salt);
+}
+
