@@ -36,6 +36,8 @@ int mcrypt_ofb(int td, void *plaintext,int len);
 int mdecrypt_ofb(int td, void *ciphertext, int len);
 int init_mcrypt_ofb( const int algorithm, void * key, int lenofkey, void *IV);
 
+void gen_key(void *keyword, void *salt, size_t salt_size, size_t given_size,
+             unsigned char *password);
 
 # define BLOWFISH 0
 # define DES 1 
